package com.logicbig.example;

import javax.validation.*;
import javax.validation.executable.ExecutableValidator;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Set;

public class MultipleValidatorMain {

    private static Validator validator;

    public static void main(String[] args) throws NoSuchMethodException {
        System.out.println("-- validating cross param --");
        validateCrossParameters(7, 2);
        validateCrossParameters(6, 2);
        validateCrossParameters(2, 8);

        System.out.println("-- validating return values --");
        validateReturnValue();
    }

    private static void validateReturnValue() throws NoSuchMethodException {
        MyBean myBean = new MyBean();
        List<Integer> returnedValued = myBean.findEvenNumbersBetween(2, 8);
        Method method = MyBean.class.getDeclaredMethod(
                "findEvenNumbersBetween", Integer.class, Integer.class);

        Validator validator = getValidator();
        ExecutableValidator executableValidator = validator.forExecutables();
        Set<ConstraintViolation<MyBean>> violations =
                executableValidator.validateReturnValue(myBean, method, returnedValued);
        if (violations.size() > 0) {
            violations.stream().forEach(MultipleValidatorMain::printError);
        } else {
            System.out.println("returned value validation passed");
            //proceed using object
        }
    }

    private static void validateCrossParameters(int x, int y) throws NoSuchMethodException {
        MyBean myBean = new MyBean();

        Method method = MyBean.class.getDeclaredMethod("findEvenNumbersBetween",
                Integer.class, Integer.class);

        Validator validator = getValidator();
        ExecutableValidator executableValidator = validator.forExecutables();
        Set<ConstraintViolation<MyBean>> violations =
                executableValidator.validateParameters(myBean, method, new Object[]{x, y});
        if (violations.size() > 0) {
            violations.stream().forEach(MultipleValidatorMain::printError);
        } else {
            System.out.println("cross parameter validation passed");
            //proceed using object
        }
    }

    private static Validator getValidator() {
        if (validator == null) {
            Configuration<?> config = Validation.byDefaultProvider().configure();
            ValidatorFactory factory = config.buildValidatorFactory();
            validator = factory.getValidator();
            factory.close();
        }
        return validator;
    }

    private static void printError(
            ConstraintViolation<MyBean> violation) {
        System.out.println(violation.getPropertyPath() + " " + violation.getMessage());
    }
}